"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamoDbCrudService = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
const q_id_generator_1 = require("q-id-generator");
const dynamoDbClient_service_1 = __importDefault(require("./dynamoDbClient.service"));
const queryBuilder_service_1 = __importDefault(require("./queryBuilder.service"));
const constants_1 = require("../../../utils/constants");
class DynamoDbCrudService {
    constructor(tableName, idColumnName = 'id', partitionKey = null) {
        this.dynamoDbClientService = new dynamoDbClient_service_1.default(tableName);
        this.tableName = tableName;
        this.idColumnName = idColumnName;
        this.partitionKey = partitionKey;
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars, no-unused-vars
    findAll(options) {
        throw new Error('Method not implemented.');
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars, no-unused-vars
    runQuery(queryCommand) {
        throw new Error('Method not implemented.');
    }
    async create(entity) {
        var _a;
        const id = ((_a = entity[this.idColumnName]) === null || _a === void 0 ? void 0 : _a.toString()) || (0, q_id_generator_1.getId)();
        const body = Object.assign(Object.assign({}, entity), { [this.idColumnName]: id });
        await this.dynamoDbClientService.put(body);
        return body;
    }
    async findById(options) {
        var _a;
        const query = new queryBuilder_service_1.default();
        this.applyFilters(query, options === null || options === void 0 ? void 0 : options.filters);
        query.limit(1);
        const command = query.get();
        const result = await this.dynamoDbClientService.query(command);
        if ((result === null || result === void 0 ? void 0 : result.Items) && ((_a = result.Items) === null || _a === void 0 ? void 0 : _a.length) > 0) {
            return result.Items[0];
        }
        if (options.throwErrorIfNull)
            throw new Error('NOT_FOUND');
        return null;
    }
    applyWhereFilter(query, filter) {
        var _a;
        const operator = constants_1.FILTER_OPERATOR_MAP[(_a = filter.operator) !== null && _a !== void 0 ? _a : constants_1.DYNAMODB_OPERATORS.EQUAL];
        query.where(filter.attribute)[operator](filter.value);
    }
    applyFilterFilter(query, filter) {
        var _a;
        const operator = constants_1.FILTER_OPERATOR_MAP[(_a = filter.operator) !== null && _a !== void 0 ? _a : constants_1.DYNAMODB_OPERATORS.EQUAL];
        query.filter(filter.attribute)[operator](filter.value);
    }
    applyFilters(query, filters) {
        if (typeof filters === 'string' && this.partitionKey) {
            query.filter(this.partitionKey)[constants_1.DYNAMODB_OPERATORS.EQUAL](filters);
        }
        else if (Array.isArray(filters)) {
            filters.forEach((filter) => {
                const isWhereProperty = [
                    this.partitionKey,
                    this.idColumnName,
                ].includes(filter.attribute);
                isWhereProperty
                    ? this.applyWhereFilter(query, filter)
                    : this.applyFilterFilter(query, filter);
            });
        }
    }
    applySorting(query, sorting, sortIndexes = []) {
        if (sorting) {
            const { field, direction } = sorting;
            const sortIndex = sortIndexes.find((sortIndex) => sortIndex.field === field);
            if (sortIndex) {
                query.usingIndex(sortIndex.index);
                if (direction === 'DESC') {
                    query.descending();
                }
                else {
                    query.ascending();
                }
            }
        }
    }
    isNotEmptyObject(obj) {
        return (obj !== null &&
            typeof obj === 'object' &&
            Object.keys(obj).length > 0);
    }
    async fetchResults(command, limit = 100) {
        var _a, _b, _c, _d, _e;
        let results = [];
        let LastEvaluatedKey = {};
        do {
            if (this.isNotEmptyObject(LastEvaluatedKey))
                command.ExclusiveStartKey = LastEvaluatedKey;
            const result = await this.dynamoDbClientService.query(command);
            command.Limit = ((_a = command.Limit) !== null && _a !== void 0 ? _a : 0) - ((_c = (_b = result.Items) === null || _b === void 0 ? void 0 : _b.length) !== null && _c !== void 0 ? _c : 0);
            results = (_d = result.Items) !== null && _d !== void 0 ? _d : [];
            LastEvaluatedKey = (_e = result.LastEvaluatedKey) !== null && _e !== void 0 ? _e : {};
        } while (results.length < limit &&
            this.isNotEmptyObject(LastEvaluatedKey));
        const lastEvaluatedKey = this.isNotEmptyObject(LastEvaluatedKey)
            ? Buffer.from(JSON.stringify(LastEvaluatedKey)).toString('base64')
            : null;
        return { items: results, lastEvaluatedKey };
    }
    applyPagination(query, pagination) {
        if (pagination === null || pagination === void 0 ? void 0 : pagination.limit)
            query.limit(pagination.limit);
        if (pagination === null || pagination === void 0 ? void 0 : pagination.from)
            query.startKey(pagination.from);
    }
    find(options = {}) {
        var _a;
        const query = new queryBuilder_service_1.default();
        // if (options.indexName) query.usingIndex(options.indexName);
        this.applyPagination(query, options === null || options === void 0 ? void 0 : options.pagination);
        this.applyFilters(query, options.filters);
        // this.applySorting(query, options.orderBy, options.sortIndexes);
        return this.fetchResults(query.get(), (_a = options === null || options === void 0 ? void 0 : options.pagination) === null || _a === void 0 ? void 0 : _a.limit);
    }
    async update(compoundFilters, data) {
        const savedRecord = await this.findById({
            filters: compoundFilters.filters,
        });
        const newData = Object.assign(Object.assign({}, savedRecord), data);
        if (this.partitionKey)
            newData[this.partitionKey] = savedRecord[this.partitionKey];
        newData[this.idColumnName] = savedRecord[this.idColumnName];
        await this.dynamoDbClientService.put(newData);
        return newData;
    }
    async remove(compoundFilters) {
        const key = compoundFilters.filters.reduce((obj, item) => {
            const property = item.attribute;
            obj[property] = item.value;
            return obj;
        }, {});
        await this.dynamoDbClientService.remove(key);
    }
}
exports.DynamoDbCrudService = DynamoDbCrudService;
//# sourceMappingURL=dynamoDbCrud.service.js.map