"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const queryBuilderCondition_service_1 = __importDefault(require("./queryBuilderCondition.service"));
const queryHelpers_1 = require("../../../helpers/queryHelpers");
class QueryBuilderService {
    constructor() {
        this.command = {};
        this.condition = new queryBuilderCondition_service_1.default(this);
    }
    get() {
        const condition = this.condition.get();
        return Object.assign(Object.assign({}, this.command), condition);
    }
    limit(num) {
        this.command.Limit = num;
        return this;
    }
    usingIndex(indexName) {
        this.command.IndexName = indexName;
        return this;
    }
    ascending() {
        this.command.ScanIndexForward = true;
        return this;
    }
    descending() {
        this.command.ScanIndexForward = false;
        return this;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    startKey(lastEvaluatedKey) {
        this.command.ExclusiveStartKey = lastEvaluatedKey;
        return this;
    }
    projection(fields) {
        const expression = [];
        fields.forEach((field) => {
            const key = this.condition.project(field);
            expression.push(key);
        });
        this.command.ProjectionExpression = expression.join(',');
        return this;
    }
    where(keyName) {
        this.condition.setKey(keyName).from(queryHelpers_1.METHOD_TO_QUERY.where);
        return this.condition;
    }
    filter(keyName) {
        this.condition.setKey(keyName).from(queryHelpers_1.METHOD_TO_QUERY.filter);
        return this.condition;
    }
    update(attribute) {
        for (const [key, value] of Object.entries(attribute)) {
            this.condition.setKey(key).from(queryHelpers_1.METHOD_TO_QUERY.update);
            this.condition.eq(value);
        }
        return this;
    }
}
exports.default = QueryBuilderService;
//# sourceMappingURL=queryBuilder.service.js.map