"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const queryHelpers_1 = require("../../../helpers/queryHelpers");
class QueryBuilderConditionService {
    constructor(query) {
        this.tempKey = '';
        this.wheres = [];
        this.filters = [];
        this.updates = [];
        this.attributeNames = {};
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        this.attributeValues = {};
        this.query = query;
        this.command = {};
    }
    get() {
        this.build();
        return this.command;
    }
    setKey(key) {
        this.tempKey = key;
        return this;
    }
    from(methodName) {
        this.queryFrom = methodName;
        return this;
    }
    eq(keyValue) {
        const { key, value } = this.generateKeyValue(keyValue);
        const expression = `${key} = ${value}`;
        this.setExpression(expression);
        return this.query;
    }
    contains(keyValue) {
        const { key, value } = this.generateKeyValue(keyValue);
        const expression = `contains(${key}, ${value})`;
        this.setExpression(expression);
        return this.query;
    }
    beginsWith(keyValue) {
        const { key, value } = this.generateKeyValue(keyValue);
        const expression = `begins_with(${key}, ${value})`;
        this.setExpression(expression);
        return this.query;
    }
    project(keyValue) {
        const key = `#${keyValue}`;
        this.attributeNames[key] = keyValue;
        return key;
    }
    setExpression(expression) {
        switch (this.queryFrom) {
            case queryHelpers_1.METHOD_TO_QUERY.filter:
                this.filters.push(expression);
                break;
            case queryHelpers_1.METHOD_TO_QUERY.update:
                this.updates.push(expression);
                break;
            default:
                this.wheres.push(expression);
                break;
        }
    }
    generateKeyValue(value) {
        const keyExpression = `#${this.tempKey}1`;
        const valueExpression = `:${this.tempKey}1`;
        this.attributeNames[keyExpression] = this.tempKey;
        this.attributeValues[valueExpression] = value;
        return { key: keyExpression, value: valueExpression };
    }
    build() {
        if (this.wheres.length > 0) {
            const keyConditionExpression = this.wheres.join(' AND ');
            this.command['KeyConditionExpression'] = keyConditionExpression;
        }
        if (this.filters.length > 0) {
            const filterExpression = this.filters.join(' AND ');
            this.command['FilterExpression'] = filterExpression;
        }
        if (this.updates.length > 0) {
            const filterExpression = this.updates.join(', ');
            this.command['UpdateExpression'] = `SET ${filterExpression}`;
        }
        if (Object.values(this.attributeNames).length > 0)
            this.command['ExpressionAttributeNames'] = this.attributeNames;
        if (Object.values(this.attributeValues).length > 0)
            this.command['ExpressionAttributeValues'] = this.attributeValues;
    }
}
exports.default = QueryBuilderConditionService;
//# sourceMappingURL=queryBuilderCondition.service.js.map