declare enum ORDER_DIRECTIONS {
    ASC = "ASC",
    DESC = "DESC"
}
declare enum FILTER_LOGIC_OPERATORS {
    AND = "AND",
    OR = "OR"
}

declare const VALID_ORDER_DIRECTIONS: ORDER_DIRECTIONS[];
type SortDirection = (typeof VALID_ORDER_DIRECTIONS)[number];

declare const VALID_FILTER_LOGIC_OPERATORS: FILTER_LOGIC_OPERATORS[];
type FilterLogicOperator = (typeof VALID_FILTER_LOGIC_OPERATORS)[number];

interface IFilter {
    attribute: string;
    operator?: string;
    value: any;
}

interface ICompoundFilter {
    filters: IFilter[] | ICompoundFilter;
    logicOperator: FilterLogicOperator;
}

interface IFindIndex {
    indexName: string;
    partitionKey?: string;
    sortKey?: string;
}

interface IFindPagination {
    limit?: number;
    from?: any;
}

interface IFindOptions {
    orderBy?: SortDirection[];
    filters?: (IFilter | ICompoundFilter)[];
    fields?: string[];
    pagination?: IFindPagination;
    throwErrorIfNull?: boolean;
    indexes?: (IFindIndex | string)[];
}

interface IFindResult<T> {
    items: T[];
    pagination?: IFindPagination | null;
}

declare class AbstractCrudService<T> {
    private crudService;
    constructor(tableName: string, idColumnName?: string, partitionKey?: string | null);
    create(entity: T): Promise<T>;
    find(options: IFindOptions): Promise<IFindResult<T>>;
    findById(options: IFindOptions): Promise<T | null>;
    update(compoundFilters: ICompoundFilter, data: Partial<T>): Promise<T | null>;
    remove(compoundFilters: ICompoundFilter): Promise<void>;
    buildFilter(attribute: string, value: string, operator?: string): {
        attribute: string;
        operator: string;
        value: string;
    };
}

export { AbstractCrudService };
